package demo.payment.controller;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;

import com.huifu.jh.core.DefaultGcbPayCallbackMessageParser;
import com.huifu.jh.core.DefaultGcbPayClient;
import com.huifu.jh.core.GcbPayCallbackMessageParser;
import com.huifu.jh.core.GcbPayClient;
import com.huifu.jh.payment.model.GcbAggPaymentProductDesc;
import com.huifu.jh.payment.model.GcbBankTransPaymentRequest;
import com.huifu.jh.payment.model.GcbBankTransPaymentResponse;
import com.huifu.jh.payment.model.GcbBankTransPaymentResult;

import cn.hutool.core.date.DateUtil;
import demo.payment.config.AppConfig;

/**
 * 
 * @author cheng.luo
 *
 * 转账支付API
 */
@Controller
public class BankTransPaymentController {
	
	private Logger logger = LoggerFactory.getLogger(getClass());
	
	@Autowired
	private AppConfig appConfig;
	
	/**
	 * 步骤一、构造支付订单
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 */
	@GetMapping("/showBankTransPaymentOrder.do")
	public String showAggPaymentOrder(HttpServletRequest request, HttpServletResponse response, Model model) {
		
		Map<String, String> orderInfoMap = this.buildBankTransPaymentOrder();
		
		model.addAllAttributes(orderInfoMap);
		
		return "bankTransPaymentOrder";
	}
	
	/**
	 * 构造演示订单信息
	 * @return
	 */
	private Map<String, String> buildBankTransPaymentOrder() {
		
		Date currentDate = Calendar.getInstance().getTime();
		
		Map<String, String> map = new HashMap<String, String>();
		map.put("bgUrl", 		"http://localhost:8080/bankTransPaymentResultBg.do"); 	//必填，商户系统接受支付结果的后台地址【url公网可以访问，否则接收不到通知】
		map.put("payerName", 		"李财水"); 	//条件必填，支付人姓名，中文名，网银直连时必填
		map.put("payerContactType", ""); 		//非必填，支付人联系方式类型，固定值：1或者2。1-代表电子邮件方式、2-其他联系方式。
		map.put("payerContact", 	""); 		//非必填，支付人联系方式，根据payerContactType的方式填写对应字符，邮箱或者其他联系方式
		map.put("payerIdentityCard","320125198805232313"); 	//条件必填，支付人身份证号码，字符串，18位身份证号；跨境结算且为直连模式时必填
		map.put("cardNumber", 		"6225881257400000"); //非必填，支付人所持卡号
		map.put("orderId", 	DateUtil.format(currentDate, "yyyyMMddHHmmssSSS")); //必填，商户订单号，只允许使用字母、数字、_,每商户提交的订单号，必须在自身账户交易中唯一
		map.put("orderCurrency", 	"CNY"); 	//必填，订单币种，3位币别码，详情请参照币别代码附录。
		map.put("settlementCurrency", "CNY"); 	//必填，结算币种，3位币别码，详情请参照币别代码附录。
		map.put("orderAmount", "100"); 	//必填，商户订单金额，整型数字，单位为分。日韩元支持的最小单位为元。如100元，则填写10000
		map.put("orderTime", DateUtil.format(currentDate, "yyyyMMddHHmmss")); 	//必填，商户订单提交时间，一共14位格式为：yyyyMMddHHmmss
		map.put("productName", "CellPhone"); 	//必填，商品名称
		map.put("productNum", "1"); 			//必填，商品数量，整型数字
		map.put("productId", "100003"); 		//必填，商品代码，请根据我司提供的商品代码附录进行填写。
		map.put("productDesc", "P0000002|iphone 6 plus|1|1|http://www.mshop.com/product?id=P0000002"); //必填，填写规则：商品编号|商品名称|商品数量|商品单价（单位元）|商品链接;商品编号|商品名称|商品数量|商品单价（单位元）|商品链接
		map.put("ext1", ""); //非必填，扩展字段1，英文或中文字符串支付完成后，按照原样返回给商户
		map.put("ext2", ""); //非必填，扩展字段2，英文或中文字符串支付完成后，按照原样返回给商户

		
		return map;
	}
	

	/**
	 * 步骤二、构建支付信息并提交到支付网关接口
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 * @throws Exception 
	 */
	@PostMapping("/doBankTransPayment.do")
	public String doPayment(HttpServletRequest request, HttpServletResponse response, Model model) throws Exception {
		
		// 转账支付API服务的url
		String serverUrl = "https://hfgj.testpnr.com/pay/banktransApi.htm";
		// 转账支付API服务的url(生产环境)
//		String serverUrl = "https://global.chinapnr.com/pay/banktransApi.htm";
		
		// 实例化客户端
		GcbPayClient gcbPayClient = new DefaultGcbPayClient(serverUrl, appConfig.getBase64PrivateKey(), appConfig.getBase64PublicKey());
		
		// 构建请求对象
		GcbBankTransPaymentRequest paymentRequest = new GcbBankTransPaymentRequest();
		
		//必填，商户系统接受支付结果的后台地址【url公网可以访问，否则接收不到通知】
		paymentRequest.setBgUrl(request.getParameter("bgUrl")); 
		//必填，会员账号，由我司提供的商户号+01，共13位数字
		paymentRequest.setMerchantAcctId(appConfig.getMerchantAcctId()); 
		//必填，终端号，由我司提供的终端号
		paymentRequest.setTerminalId(appConfig.getTerminalId());	
		//条件必填，支付人姓名，中文名，网银直连时必填
		paymentRequest.setPayerName(request.getParameter("payerName")); 
		 //条件必填，支付人身份证号码，字符串，18位身份证号；跨境结算且为直连模式时必填
		paymentRequest.setPayerIdentityCard(request.getParameter("payerIdentityCard"));
		//非必填，支付人所持卡号
		paymentRequest.setCardNumber(request.getParameter("cardNumber")); 
		//必填，商户订单号，只允许使用字母、数字、_,每商户提交的订单号，必须在自身账户交易中唯一
		paymentRequest.setOrderId(request.getParameter("orderId")); 
		//必填，订单币种，3位币别码，详情请参照币别代码附录。
		paymentRequest.setOrderCurrency(request.getParameter("orderCurrency")); 
		//必填，结算币种，3位币别码，详情请参照币别代码附录。
		paymentRequest.setSettlementCurrency(request.getParameter("settlementCurrency"));
		//必填，商户订单金额，整型数字，单位为分。日韩元支持的最小单位为元。如100元，则填写10000
		paymentRequest.setOrderAmount(request.getParameter("orderAmount")); 
		//必填，商户订单提交时间，一共14位格式为：yyyyMMddHHmmss
		paymentRequest.setOrderTime(request.getParameter("orderTime")); 
		//必填，商品名称
		paymentRequest.setProductName(request.getParameter("productName")); 
		//必填，商品数量，整型数字
		paymentRequest.setProductNum(request.getParameter("productNum")); 
		//必填，商品代码，请根据我司提供的商品代码附录进行填写。
		paymentRequest.setProductId(request.getParameter("productId")); 
		//必填，填写规则：商品编号|商品名称|商品数量|商品单价（单位元）|商品链接;商品编号|商品名称|商品数量|商品单价（单位元）|商品链接
		paymentRequest.setProductDescList(this.buildProductDescList(request.getParameter("productDesc")));
		//非必填，扩展字段1，英文或中文字符串支付完成后，按照原样返回给商户
		paymentRequest.setExt1(request.getParameter("ext1"));
		//非必填，扩展字段2，英文或中文字符串支付完成后，按照原样返回给商户
		paymentRequest.setExt2(request.getParameter("ext2"));
		
		//必填，1：pc端支付2：移动端支付
		paymentRequest.setDeviceType("1"); 
		//必填，F：转账支付（API）
		paymentRequest.setPayType("F");  
		//非必填，1:表示同一订单禁止重复提交标志，预留字段。2:表示 同一订单，可重复提交（订单状态为未支付成功）
		paymentRequest.setRedoFlag("1");
		//非必填，1:表示分账，不上送或上送。0:表示不分账；分账需要联系我司运营进行开通配置
		paymentRequest.setDivFlag("0"); 
		// 风控信息（JSON格式），留学、航空、酒店类需按要求上送。
		paymentRequest.setRiskInfo("{\"logisticsInfo\":{\"subProductCode\":\"LOG\"}}");
		
		// 发送请求
		GcbBankTransPaymentResponse paymentResponse = gcbPayClient.execute(paymentRequest);
		logger.info("GcbBankTransPaymentResponse => " + new String(paymentResponse.getResponseBody()));
		
		// 商户业务逻辑处理
		// ......
		
		model.addAttribute("resultMap", paymentResponse.getParamMap());
		
		return "bankTransPaymentResult";
	}
	
	/**
	 * 构造产品描述信息列表
	 * @param productDescStr
	 * @return
	 */
	private List<GcbAggPaymentProductDesc> buildProductDescList(String productDescStr) {
		if (StringUtils.isBlank(productDescStr)) {
			return null;
		}
		
		List<GcbAggPaymentProductDesc> descList = new ArrayList<GcbAggPaymentProductDesc>();
		String[] descArray = productDescStr.trim().split(";");
		for (String desc : descArray) {
			String[] fieldArray = desc.split("|");
			
			GcbAggPaymentProductDesc descObj = new GcbAggPaymentProductDesc();
			descObj.setCode(fieldArray[0]);
			descObj.setName(fieldArray[1]);
			descObj.setQuantity(fieldArray[2]);
			descObj.setPrice(fieldArray[3]);
			descObj.setUrl(fieldArray[4]);
			
			descList.add(descObj);
		}
		
		return descList;
	}
	
	
	
	/**
	 * 步骤三、处理bg支付结果通知
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 * @throws Exception 
	 */
	@RequestMapping("/bankTransPaymentResultBg.do")
	public String processBgNotification(HttpServletRequest request, HttpServletResponse response, Model model) throws Exception {
		
		// 实例化通知解析器
		GcbPayCallbackMessageParser gcbMessageParser = new DefaultGcbPayCallbackMessageParser(appConfig.getBase64PublicKey());
		
		// 解析通知结果
		GcbBankTransPaymentResult result = gcbMessageParser.parse(request, GcbBankTransPaymentResult.class);
		logger.info("GcbBankTransPaymentResult =>" + result.getData());
		
		// 根据通知结果进行业务逻辑处理
		// ......
		
		
		// 向服务端返回: <result>1</result> ,表示结果接收成功，无需再通知。
		response.getWriter().write("<result>1</result>");
		
		return null;
	}
}
