package demo.payment.controller;

import java.io.File;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.io.FileUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;

import com.huifu.jh.core.DefaultGcbPayClient;
import com.huifu.jh.core.GcbPayClient;
import com.huifu.jh.payment.model.GcbOneDayPaymentDealsDownloadRequest;
import com.huifu.jh.payment.model.GcbOneDayPaymentDealsDownloadResponse;

import cn.hutool.core.date.DateUtil;
import demo.payment.config.AppConfig;

/**
 * 
 * @author cheng.luo
 * 
 * 日终交易查询结果下载
 *
 */
@Controller
public class OneDayPaymentDealsDownloadController {
	
	private Logger logger = LoggerFactory.getLogger(getClass());
	
	@Autowired
	private AppConfig appConfig;
	
	/**
	 * 步骤一、构造日终交易查询结果下载条件
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 */
	@GetMapping("/showOneDayPaymentDealsDownloadCond.do")
	public String showOneDayPaymentDealsDownloadCond(HttpServletRequest request, HttpServletResponse response, Model model) {
		
		Map<String, String> orderInfoMap = this.buildOneDayPaymentDealsDownloadCond();
		
		model.addAllAttributes(orderInfoMap);
		
		return "oneDayPaymentDealsDownloadCond";
	}
	
	/**
	 * 构造演示信息
	 * @return
	 */
	private Map<String, String> buildOneDayPaymentDealsDownloadCond() {
		
		Map<String, String> map = new HashMap<String, String>();
		return map;
	}
	

	/**
	 * 步骤二、构建查询条件并提交到服务接口
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 * @throws Exception 
	 */
	@PostMapping("/doOneDayPaymentDealsDownload.do")
	public String doPaymentDealQuery(HttpServletRequest request, HttpServletResponse response, Model model) throws Exception {
		
		// 日终交易明细文件下载服务的url
		String serverUrl = "https://hfgj.testpnr.com/pay/dailyFileDownload.htm";
		//日终交易明细文件下载服务的url(生产环境)
//		String serverUrl = "https://global.chinapnr.com/pay/dailyFileDownload.htm";
		
		// 实例化客户端
		GcbPayClient gcbPayClient = new DefaultGcbPayClient(serverUrl, appConfig.getBase64PrivateKey(), appConfig.getBase64PublicKey());
		
		// 构建请求对象
		GcbOneDayPaymentDealsDownloadRequest downloadRequest = new GcbOneDayPaymentDealsDownloadRequest();
		
		//必填，会员账号，由我司提供的商户号+01，共13位数字
		downloadRequest.setMerchantAcctId(appConfig.getMerchantAcctId()); 
		//必填，终端号，由我司提供的终端号
		downloadRequest.setTerminalId(appConfig.getTerminalId());
		//必填，唯一关联号,由我司提供。调用日终交易查询接口返回的token
		downloadRequest.setToken(request.getParameter("token"));
		
		// 发送请求
		GcbOneDayPaymentDealsDownloadResponse downloadResponse = gcbPayClient.execute(downloadRequest);
		
		// 商户业务逻辑处理
		// ......
		
		// 交易明细压缩文件字节流，注：压缩文件格式为zip
		byte[] fileContent = downloadResponse.getFileContent();
		if (fileContent != null && fileContent.length > 0) {
			// 保存文件
			String fileName = DateUtil.format(DateUtil.date(), "yyyyMMddHHmmssSSS") + ".zip";
			String filePath = FileUtils.getTempDirectoryPath() + fileName;
			logger.info("tempFilePath=>" + filePath);
			FileUtils.writeByteArrayToFile(new File(filePath), fileContent);
			
			// ......
		} else {
			logger.info("GcbOneDayPaymentDealsDownloadResponse => " +new String(downloadResponse.getResponseBody()));
		}
		
		int fileSize = (downloadResponse.getFileContent() == null) ? 0 : downloadResponse.getFileContent().length;
		downloadResponse.getParamMap().put("fileSize", fileSize+"");
		model.addAttribute("resultMap", downloadResponse.getParamMap());
		return "paymentDealQueryResult";
	}
}
