package demo.payment.controller;

import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;

import com.huifu.jh.core.DefaultGcbPayClient;
import com.huifu.jh.core.GcbPayClient;
import com.huifu.jh.payment.model.GcbOneDayPaymentDealsQueryRequest;
import com.huifu.jh.payment.model.GcbOneDayPaymentDealsQueryResponse;

import demo.payment.config.AppConfig;

/**
 * 
 * @author cheng.luo
 * 
 * 日终交易查询
 *
 */
@Controller
public class OneDayPaymentDealsQueryController {
	
	private Logger logger = LoggerFactory.getLogger(getClass());
	
	@Autowired
	private AppConfig appConfig;
	
	/**
	 * 步骤一、构造日终交易查询条件
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 */
	@GetMapping("/showOneDayPaymentDealsQueryCond.do")
	public String showOneDayPaymentDealsQueryCond(HttpServletRequest request, HttpServletResponse response, Model model) {
		
		Map<String, String> orderInfoMap = this.buildOneDayPaymentDealsQueryCond();
		
		model.addAllAttributes(orderInfoMap);
		
		return "oneDayPaymentDealsQueryCond";
	}
	
	/**
	 * 构造演示信息
	 * @return
	 */
	private Map<String, String> buildOneDayPaymentDealsQueryCond() {
		
		Map<String, String> map = new HashMap<String, String>();
		map.put("trxDate", 		"20210930");  	//必填，数字串，一共8位格式为：yyyyMMdd
		return map;
	}
	

	/**
	 * 步骤二、构建查询条件并提交到服务接口
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 * @throws Exception 
	 */
	@PostMapping("/doOneDayPaymentDealsQuery.do")
	public String doOneDayPaymentDealsQuery(HttpServletRequest request, HttpServletResponse response, Model model) throws Exception {
		
		// 日终交易明细查询服务的url
		String serverUrl = "https://hfgj.testpnr.com/pay/dailyTxnConfirm.htm";
		// 日终交易明细查询服务的url(生产环境)
//		String serverUrl = "https://global.chinapnr.com/pay/dailyTxnConfirm.htm";
		
		// 实例化客户端
		GcbPayClient gcbPayClient = new DefaultGcbPayClient(serverUrl, appConfig.getBase64PrivateKey(), appConfig.getBase64PublicKey());
		
		// 构建请求对象
		GcbOneDayPaymentDealsQueryRequest queryRequest = new GcbOneDayPaymentDealsQueryRequest();
		
		//必填，会员账号，由我司提供的商户号+01，共13位数字
		queryRequest.setMerchantAcctId(appConfig.getMerchantAcctId()); 
		//必填，终端号，由我司提供的终端号
		queryRequest.setTerminalId(appConfig.getTerminalId());
		//必填，交易日，数字串，一共8位格式为：yyyyMMdd
		queryRequest.setTrxDate(request.getParameter("trxDate"));		
		// 发送请求
		GcbOneDayPaymentDealsQueryResponse queryResponse = gcbPayClient.execute(queryRequest);
		logger.info("GcbOneDayPaymentDealsQueryResponse => " + new String(queryResponse.getResponseBody()));
		
		// 商户业务逻辑处理
		// ......
		
		model.addAttribute("resultMap", queryResponse.getParamMap());
		return "oneDayPaymentDealsQueryResult";
	}
}
