package demo.payment.controller;

import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;

import com.huifu.jh.core.DefaultGcbPayCallbackMessageParser;
import com.huifu.jh.core.DefaultGcbPayClient;
import com.huifu.jh.core.GcbPayCallbackMessageParser;
import com.huifu.jh.core.GcbPayClient;
import com.huifu.jh.payment.model.GcbPaymentDealCloseRequest;
import com.huifu.jh.payment.model.GcbPaymentDealCloseResponse;
import com.huifu.jh.payment.model.GcbPaymentDealCloseResult;

import demo.payment.config.AppConfig;

/**
 * 
 * @author cheng.luo
 *
 * 交易关单申请
 */
@Controller
public class PaymentDealCloseApplyController {
	
	private Logger logger = LoggerFactory.getLogger(getClass());
	
	@Autowired
	private AppConfig appConfig;
	
	/**
	 * 步骤一、构造关单申请信息
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 */
	@GetMapping("/showPaymentDealCloseApplyInfo.do")
	public String showPaymentDealDivApplyInfo(HttpServletRequest request, HttpServletResponse response, Model model) {
		
		Map<String, String> orderInfoMap = this.buildPaymentDealCloseApplyInfo();
		
		model.addAllAttributes(orderInfoMap);
		
		return "paymentDealCloseApplyInfo";
	}
	
	/**
	 * 构造演示关单申请信息
	 * @return
	 */
	private Map<String, String> buildPaymentDealCloseApplyInfo() {
		
		Map<String, String> map = new HashMap<String, String>();
		map.put("bgUrl", "http://localhost:8080/paymentCloseResultBg.do"); 	//非必填，服务器接受关单结果的后台地址商户不传不会通知,bgUrl地址公网必须能访问
		map.put("orderId", "9920211021185047317"); 		//必填，原交易订单号
		
		return map;
	}
	

	/**
	 * 步骤二、构建关单申请信息并提交到网关接口
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 * @throws Exception 
	 */
	@PostMapping("/doPaymentDealCloseApply.do")
	public String doPaymentDealCloseApply(HttpServletRequest request, HttpServletResponse response, Model model) throws Exception {
		
		// 关单申请服务的url
		String serverUrl = "https://hfgj.testpnr.com/pay/unifiedclose.htm";
		// 关单申请服务的url(生产环境)
//		String serverUrl = "https://global.chinapnr.com/pay/unifiedclose.htm";
		
		// 实例化客户端
		GcbPayClient gcbPayClient = new DefaultGcbPayClient(serverUrl, appConfig.getBase64PrivateKey(), appConfig.getBase64PublicKey());
		
		// 构建请求对象
		GcbPaymentDealCloseRequest applyRequest = new GcbPaymentDealCloseRequest();
		
		//必填，会员账号，由我司提供的商户号+01，共13位数字
		applyRequest.setMerchantAcctId(appConfig.getMerchantAcctId()); 
		//必填，终端号，由我司提供的终端号
		applyRequest.setBgUrl(request.getParameter("bgUrl"));
		//必填，请求流水号,只允许使用字母、数字、_，每个商户提交的流水号，必须在自身账户交易中唯一
		applyRequest.setOrderId(request.getParameter("orderId"));
		
		// 发送请求
		GcbPaymentDealCloseResponse applyResponse = gcbPayClient.execute(applyRequest);
		logger.info("GcbPaymentDealCloseResponse => " + new String(applyResponse.getResponseBody()));
		
		// 商户业务逻辑处理
		// ......
		
		model.addAttribute("resultMap", applyResponse.getParamMap());
		return "paymentDealCloseApplyResult";
	}
	
	
	/**
	 * 步骤三、处理bg关单结果通知
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 * @throws Exception 
	 */
	@RequestMapping("/paymentCloseResultBg.do")
	public String processBgNotification(HttpServletRequest request, HttpServletResponse response, Model model) throws Exception {
		
		// 实例化通知解析器
		GcbPayCallbackMessageParser gcbMessageParser = new DefaultGcbPayCallbackMessageParser(appConfig.getBase64PublicKey());
		
		// 解析通知结果
		GcbPaymentDealCloseResult result = gcbMessageParser.parse(request, GcbPaymentDealCloseResult.class);
		logger.info("GcbPaymentDealCloseResult =>" + result.getData());
		
		// 根据通知结果进行业务逻辑处理
		// ......
		
		
		// 向服务端返回: <result>1</result> ,表示结果接收成功，无需再通知。
		response.getWriter().write("<result>1</result>");
		
		return null;
	}
}
