package demo.payment.controller;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;

import com.huifu.jh.core.DefaultGcbPayClient;
import com.huifu.jh.core.GcbPayClient;
import com.huifu.jh.payment.model.GcbPaymentDealDivAmtQueryRequest;
import com.huifu.jh.payment.model.GcbPaymentDealDivAmtQueryResponse;

import demo.payment.config.AppConfig;

/**
 * 
 * @author cheng.luo
 *
 * 待分账查询
 */
@Controller
public class PaymentDealDivAmtQueryController {
	
	private Logger logger = LoggerFactory.getLogger(getClass());
	
	@Autowired
	private AppConfig appConfig;
	
	/**
	 * 步骤一、构造待分账查询条件
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 */
	@GetMapping("/showPaymentDealDivAmtQueryCond.do")
	public String showPaymentDealDivAmtQueryCond(HttpServletRequest request, HttpServletResponse response, Model model) {
		
		Map<String, String> orderInfoMap = this.buildPaymentDealDivAmtQueryCond();
		
		model.addAllAttributes(orderInfoMap);
		
		return "paymentDealDivAmtQueryCond";
	}
	
	/**
	 * 构造演示待分账查询条件信息
	 * @return
	 */
	private Map<String, String> buildPaymentDealDivAmtQueryCond() {
		
		Map<String, String> map = new HashMap<String, String>();
		String requestId = new SimpleDateFormat("yyyyMMddHHmmssSSS").format(new Date());
		map.put("requestId", 		requestId);  	//必填，请求流水号,只允许使用字母、数字、_，每个商户提交的流水号，必须在自身账户交易中唯一
		
		String requestTime = new SimpleDateFormat("yyyyMMddHHmmss").format(new Date());
		map.put("requestTime", 		requestTime); 	//必填，请求时间, 数字串，一共14位。 格式：yyyyMMddHHmmss
		
		map.put("dealId", 		"1110000363965"); 	//必填，支付成功时返回的汇付支付交易唯一流水号
		map.put("ext1", ""); //非必填，扩展字段1，英文或中文字符串支付完成后，按照原样返回给商户
		map.put("ext2", ""); //非必填，扩展字段2，英文或中文字符串支付完成后，按照原样返回给商户

		
		return map;
	}
	

	/**
	 * 步骤二、构建待分账查询条件并提交到网关接口
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 * @throws Exception 
	 */
	@PostMapping("/doPaymentDealDivAmtQuery.do")
	public String doPaymentDealDivAmtQuery(HttpServletRequest request, HttpServletResponse response, Model model) throws Exception {
		
		// 分账交易结果查询服务的url
		String serverUrl = "https://hfgj.testpnr.com/ledger/unsplitQuery.htm";
		// 分账交易结果查询服务的url(生产环境)
//		String serverUrl = "https://global.chinapnr.com/ledger/unsplitQuery.htm";
		
		// 实例化客户端
		GcbPayClient gcbPayClient = new DefaultGcbPayClient(serverUrl, appConfig.getBase64PrivateKey(), appConfig.getBase64PublicKey());
		
		// 构建请求对象
		GcbPaymentDealDivAmtQueryRequest queryRequest = new GcbPaymentDealDivAmtQueryRequest();
		
		//必填，会员账号，由我司提供的商户号+01，共13位数字
		queryRequest.setMerchantAcctId(appConfig.getMerchantAcctId()); 
		//必填，终端号，由我司提供的终端号
		queryRequest.setTerminalId(appConfig.getTerminalId());
		//必填，请求流水号,只允许使用字母、数字、_，每个商户提交的流水号，必须在自身账户交易中唯一
		queryRequest.setRequestId(request.getParameter("requestId"));
		//必填，请求时间, 数字串，一共14位。 格式：yyyyMMddHHmmss
		queryRequest.setRequestTime(request.getParameter("requestTime"));
		//必填，支付成功时返回的汇付支付交易唯一流水号
		queryRequest.setDealId(request.getParameter("dealId"));
		//非必填，扩展字段1, 预留
		queryRequest.setExt1(request.getParameter("ext1"));
		//非必填，扩展字段2, 预留
		queryRequest.setExt2(request.getParameter("ext2"));
		
		// 发送请求
		GcbPaymentDealDivAmtQueryResponse queryResponse = gcbPayClient.execute(queryRequest);
		logger.info("GcbPaymentDealDivAmtQueryResponse => " + new String(queryResponse.getResponseBody()));
		
		// 商户业务逻辑处理
		// ......
		
		model.addAttribute("resultMap", queryResponse.getParamMap());
		return "paymentDealDivAmtQueryResult";
	}
	
}
