package demo.payment.controller;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;

import com.huifu.jh.core.DefaultGcbPayClient;
import com.huifu.jh.core.GcbPayClient;
import com.huifu.jh.payment.model.GcbPaymentDealDivApplyDetail;
import com.huifu.jh.payment.model.GcbPaymentDealDivApplyRequest;
import com.huifu.jh.payment.model.GcbPaymentDealDivApplyResponse;

import demo.payment.config.AppConfig;

/**
 * 
 * @author cheng.luo
 *
 * 交易分账申请
 */
@Controller
public class PaymentDealDivApplyController {
	
	private Logger logger = LoggerFactory.getLogger(getClass());
	
	@Autowired
	private AppConfig appConfig;
	
	/**
	 * 步骤一、构造分账申请信息
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 */
	@GetMapping("/showPaymentDealDivApplyInfo.do")
	public String showPaymentDealDivApplyInfo(HttpServletRequest request, HttpServletResponse response, Model model) {
		
		Map<String, String> orderInfoMap = this.buildPaymentDealDivApplyInfo();
		
		model.addAllAttributes(orderInfoMap);
		
		return "paymentDealDivApplyInfo";
	}
	
	/**
	 * 构造演示分账申请信息
	 * @return
	 */
	private Map<String, String> buildPaymentDealDivApplyInfo() {
		
		Map<String, String> map = new HashMap<String, String>();
		String requestId = new SimpleDateFormat("yyyyMMddHHmmssSSS").format(new Date());
		map.put("requestId", 		requestId);  	//必填，请求流水号,只允许使用字母、数字、_，每个商户提交的流水号，必须在自身账户交易中唯一
		
		String requestTime = new SimpleDateFormat("yyyyMMddHHmmss").format(new Date());
		map.put("requestTime", 		requestTime); 	//必填，请求时间, 数字串，一共14位。 格式：yyyyMMddHHmmss
		
		map.put("dealId", 		"1110000363965"); 	//必填，收单系统交易号，该交易在收单系统对应的交易号。并且交易是支持分账的交易，即交易的DivFlag=1
		map.put("details", "10000001|100|desc1;10000002|200|desc2"); 		//必填，分账明细, 明细笔数最多99。如：10019671|10000|描述1;1001962|20000|描述2
		map.put("ext1", ""); //非必填，扩展字段1，英文或中文字符串支付完成后，按照原样返回给商户
		map.put("ext2", ""); //非必填，扩展字段2，英文或中文字符串支付完成后，按照原样返回给商户

		
		return map;
	}
	

	/**
	 * 步骤二、构建分账申请信息并提交到网关接口
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 * @throws Exception 
	 */
	@PostMapping("/doPaymentDealDivApply.do")
	public String doPaymentDealDivApply(HttpServletRequest request, HttpServletResponse response, Model model) throws Exception {
		
		// 交易分账申请服务的url
		String serverUrl = "https://hfgj.testpnr.com/ledger/transDiv.htm";
		// 交易分账申请服务的url(生产环境)
//		String serverUrl = "https://global.chinapnr.com/ledger/transDiv.htm";
		
		// 实例化客户端
		GcbPayClient gcbPayClient = new DefaultGcbPayClient(serverUrl, appConfig.getBase64PrivateKey(), appConfig.getBase64PublicKey());
		
		// 构建请求对象
		GcbPaymentDealDivApplyRequest applyRequest = new GcbPaymentDealDivApplyRequest();
		
		//必填，会员账号，由我司提供的商户号+01，共13位数字
		applyRequest.setMerchantAcctId(appConfig.getMerchantAcctId()); 
		//必填，终端号，由我司提供的终端号
		applyRequest.setTerminalId(appConfig.getTerminalId());
		//必填，请求流水号,只允许使用字母、数字、_，每个商户提交的流水号，必须在自身账户交易中唯一
		applyRequest.setRequestId(request.getParameter("requestId"));
		//必填，请求时间, 数字串，一共14位。 格式：yyyyMMddHHmmss
		applyRequest.setRequestTime(request.getParameter("requestTime"));
		//必填，收单系统交易号，该交易在收单系统对应的交易号。并且交易是支持分账的交易，即交易的DivFlag=1
		applyRequest.setDealId(request.getParameter("dealId"));
		//必填，分账明细, 明细笔数最多99。如：10019671|10000|描述1;1001962|20000|描述2
		applyRequest.setDetailList(this.buildDetailList(request.getParameter("details")));
		//非必填，扩展字段1, 预留
		applyRequest.setExt1(request.getParameter("ext1"));
		//非必填，扩展字段2, 预留
		applyRequest.setExt2(request.getParameter("ext2"));
		
		// 发送请求
		GcbPaymentDealDivApplyResponse applyResponse = gcbPayClient.execute(applyRequest);
		logger.info("GcbPaymentDealDivApplyResponse => " + new String(applyResponse.getResponseBody()));
		
		// 商户业务逻辑处理
		// ......
		
		model.addAttribute("resultMap", applyResponse.getParamMap());
		return "paymentDealDivApplyResult";
	}
	
	/**
	 * 构造分账明细信息列表
	 * @param productDescStr
	 * @return
	 */
	private List<GcbPaymentDealDivApplyDetail> buildDetailList(String detailStr) {
		if (StringUtils.isBlank(detailStr)) {
			return null;
		}
		
		List<GcbPaymentDealDivApplyDetail> detailList = new ArrayList<GcbPaymentDealDivApplyDetail>();
		String[] detailArray = detailStr.trim().split(";");
		for (String detail : detailArray) {
			String[] fieldArray = detail.split("\\|");
			
			GcbPaymentDealDivApplyDetail detailObj = new GcbPaymentDealDivApplyDetail();
			detailObj.setUserId(fieldArray[0]);
			detailObj.setAmount(fieldArray[1]);
			detailObj.setDesc(fieldArray[2]);
			
			detailList.add(detailObj);
		}
		
		return detailList;
	}
}
