package demo.payment.controller;

import com.huifu.jh.core.DefaultGcbPayClient;
import com.huifu.jh.core.GcbPayClient;
import com.huifu.jh.payment.model.GcbPaymentDealDivCloseRequest;
import com.huifu.jh.payment.model.GcbPaymentDealDivCloseResponse;
import demo.payment.config.AppConfig;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * 分账完结申请
 */
@Controller
public class PaymentDealDivCloseController {

    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    private AppConfig appConfig;

    /**
     * 步骤一、构造分账完结申请条件
     * @param request
     * @param response
     * @param model
     * @return
     */
    @GetMapping("/showPaymentDealDivCloseCond.do")
    public String showPaymentDealDivCloseCond(HttpServletRequest request, HttpServletResponse response, Model model) {

        Map<String, String> orderInfoMap = this.buildPaymentDealDivCloseCond();

        model.addAllAttributes(orderInfoMap);

        return "paymentDealDivCloseCond";
    }

    /**
     * 构造演示分账完结申请条件信息
     * @return
     */
    private Map<String, String> buildPaymentDealDivCloseCond() {
        Map<String, String> map = new HashMap<String, String>();
        String requestId = new SimpleDateFormat("yyyyMMddHHmmssSSS").format(new Date());
        map.put("requestId", requestId); //必填，分账完结请求流水号,只允许使用字母、数字、_、-，每个商户提交的流水号，必须在自身商户号下唯一

        map.put("dealId", ""); //必填，目标交易号，对应需要执行分账完结的原支付交易的汇付交易号，可在支付流程的返回中获取。

        String requestTime = new SimpleDateFormat("yyyyMMddHHmmss").format(new Date());
        map.put("requestTime", requestTime); //必填，请求时间, 数字串，一共14位。 格式：yyyyMMddHHmmss

        map.put("ext1", ""); //非必填，扩展字段1, 预留
        map.put("ext2", ""); //非必填，扩展字段2, 预留

        return map;

    }

    /**
     * 步骤二、构建分账完结申请条件并提交到网关接口
     * @param request
     * @param response
     * @param model
     * @return
     * @throws Exception
     */
    @PostMapping("/doPaymentDealDivClose.do")
    public String doPaymentDealDivClose(HttpServletRequest request, HttpServletResponse response, Model model) throws Exception {

        // 分账完结申请服务的url
        String serverUrl = "https://hfgj.testpnr.com/ledger/close.htm";
        // 分账完结申请服务的url(生产环境)
//		String serverUrl = "https://global.chinapnr.com/ledger/close.htm";

        // 实例化客户端
        GcbPayClient gcbPayClient = new DefaultGcbPayClient(serverUrl, appConfig.getBase64PrivateKey(), appConfig.getBase64PublicKey());

        // 构建请求对象
        GcbPaymentDealDivCloseRequest closeRequest = new GcbPaymentDealDivCloseRequest();

        //必填，会员账号，由我司提供的商户号+01，共13位数字
        closeRequest.setMerchantAcctId(appConfig.getMerchantAcctId());
        //必填，终端号，由我司提供的终端号
        closeRequest.setTerminalId(appConfig.getTerminalId());
        //必填，分账完结请求流水号,只允许使用字母、数字、_、-，每个商户提交的流水号，必须在自身商户号下唯一
        closeRequest.setRequestId(request.getParameter("requestId"));
        //必填，目标交易号，对应需要执行分账完结的原支付交易的汇付交易号，可在支付流程的返回中获取。
        closeRequest.setDealId(request.getParameter("dealId"));
        //必填，请求时间, 数字串，一共14位。 格式：yyyyMMddHHmmss
        closeRequest.setRequestTime(request.getParameter("requestTime"));
        //非必填，扩展字段1, 预留
        closeRequest.setExt1(request.getParameter("ext1"));
        //非必填，扩展字段2, 预留
        closeRequest.setExt2(request.getParameter("ext2"));

        // 发送请求
        GcbPaymentDealDivCloseResponse closeResponse = gcbPayClient.execute(closeRequest);
        logger.info("GcbPaymentDealDivCloseResponse => " + new String(closeResponse.getResponseBody()));

        // 商户业务逻辑处理
        // ......

        model.addAttribute("resultMap", closeResponse.getParamMap());
        return "paymentDealDivCloseResult";
    }


}



