package demo.payment.controller;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;

import com.huifu.jh.core.DefaultGcbPayClient;
import com.huifu.jh.core.GcbPayClient;
import com.huifu.jh.payment.model.GcbPaymentDealDivRollbackApplyRequest;
import com.huifu.jh.payment.model.GcbPaymentDealDivRollbackApplyResponse;

import demo.payment.config.AppConfig;

/**
 * 
 * @author cheng.luo
 *
 * 分账回退申请
 */
@Controller
public class PaymentDealDivRollbackApplyController {
	
	private Logger logger = LoggerFactory.getLogger(getClass());
	
	@Autowired
	private AppConfig appConfig;
	
	/**
	 * 步骤一、构造分账回退信息
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 */
	@GetMapping("/showPaymentDealDivRollbackApplyInfo.do")
	public String showPaymentDealDivRollbackApplyInfo(HttpServletRequest request, HttpServletResponse response, Model model) {
		
		Map<String, String> orderInfoMap = this.buildPaymentDealDivRollbackApplyInfo();
		
		model.addAllAttributes(orderInfoMap);
		
		return "paymentDealDivRollbackApplyInfo";
	}
	
	/**
	 * 构造演示分账回退信息
	 * @return
	 */
	private Map<String, String> buildPaymentDealDivRollbackApplyInfo() {
		
		Map<String, String> map = new HashMap<String, String>();
		
		//必填，分账回退请求流水号,只允许使用字母、数字、_、-，每个商户提交的流水号，必须在自身商户号下唯一
		String returnReqId = new SimpleDateFormat("yyyyMMddHHmmssSSS").format(new Date());
		map.put("returnReqId", 		returnReqId);
		//必填，请求时间, 数字串，一共14位。 格式：yyyyMMddHHmmss
		String requestTime = new SimpleDateFormat("yyyyMMddHHmmss").format(new Date());
		map.put("requestTime", 		requestTime);
		map.put("origReqId", 		"20211020182237500"); //必填，原分账指令的请求流水号
		map.put("userId", 		"10000001"); //必填，需要回退的原分账接收方，即资金是从该用户返还
		map.put("amount", "100"); //必填，CNY，单位为分，不能超过原始分给该用户的总金额。
		map.put("ext1", ""); //非必填，扩展字段1, 预留
		map.put("ext2", ""); //非必填，扩展字段2, 预留
		
		return map;
	}
	

	/**
	 * 步骤二、构建分账回退信息并提交到网关接口
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 * @throws Exception 
	 */
	@PostMapping("/doPaymentDealDivRollbackApply.do")
	public String doPaymentDealDivRollbackApply(HttpServletRequest request, HttpServletResponse response, Model model) throws Exception {
		
		// 交易分账回退服务的url
		String serverUrl = "https://hfgj.testpnr.com/ledger/rollback.htm";
		// 交易分账回退服务的url(生产环境)
//		String serverUrl = "https://global.chinapnr.com/ledger/rollback.htm";
		
		// 实例化客户端
		GcbPayClient gcbPayClient = new DefaultGcbPayClient(serverUrl, appConfig.getBase64PrivateKey(), appConfig.getBase64PublicKey());
		
		// 构建请求对象
		GcbPaymentDealDivRollbackApplyRequest applyRequest = new GcbPaymentDealDivRollbackApplyRequest();
		
		//必填，会员账号，由我司提供的商户号+01，共13位数字
		applyRequest.setMerchantAcctId(appConfig.getMerchantAcctId()); 
		//必填，终端号，由我司提供的终端号
		applyRequest.setTerminalId(appConfig.getTerminalId());
		//必填，分账回退请求流水号,只允许使用字母、数字、_、-，每个商户提交的流水号，必须在自身商户号下唯一
		applyRequest.setReturnReqId(request.getParameter("returnReqId"));
		//必填，请求时间, 数字串，一共14位。 格式：yyyyMMddHHmmss
		applyRequest.setRequestTime(request.getParameter("requestTime"));
		//必填，原分账指令的请求流水号
		applyRequest.setOrigReqId(request.getParameter("origReqId"));
		//必填，需要回退的原分账接收方，即资金是从该用户返还
		applyRequest.setUserId(request.getParameter("userId"));
		//必填，CNY，单位为分，不能超过原始分给该用户的总金额。
		applyRequest.setAmount(request.getParameter("amount"));
		//非必填，扩展字段1, 预留
		applyRequest.setExt1(request.getParameter("ext1"));
		//非必填，扩展字段2, 预留
		applyRequest.setExt2(request.getParameter("ext2"));
		
		// 发送请求
		GcbPaymentDealDivRollbackApplyResponse applyResponse = gcbPayClient.execute(applyRequest);
		logger.info("GcbPaymentDealDivRollbackApplyResponse => " + new String(applyResponse.getResponseBody()));
		
		// 商户业务逻辑处理
		// ......
		
		model.addAttribute("resultMap", applyResponse.getParamMap());
		return "paymentDealDivRollbackApplyResult";
	}
}
