package demo.payment.controller;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;

import com.huifu.jh.core.DefaultGcbPayClient;
import com.huifu.jh.core.GcbPayClient;
import com.huifu.jh.payment.model.GcbPaymentDealDivRollbackQueryRequest;
import com.huifu.jh.payment.model.GcbPaymentDealDivRollbackQueryResponse;

import demo.payment.config.AppConfig;

/**
 * 
 * @author cheng.luo
 *
 * 分账回退结果查询
 */
@Controller
public class PaymentDealDivRollbackQueryController {
	
	private Logger logger = LoggerFactory.getLogger(getClass());
	
	@Autowired
	private AppConfig appConfig;
	
	/**
	 * 步骤一、构造分账回退查询条件
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 */
	@GetMapping("/showPaymentDealDivRollbackQueryCond.do")
	public String showPaymentDealDivRollbackQueryCond(HttpServletRequest request, HttpServletResponse response, Model model) {
		
		Map<String, String> orderInfoMap = this.buildPaymentDealDivRollbackQueryCond();
		
		model.addAllAttributes(orderInfoMap);
		
		return "paymentDealDivRollbackQueryCond";
	}
	
	/**
	 * 构造演示分账回退查询条件信息
	 * @return
	 */
	private Map<String, String> buildPaymentDealDivRollbackQueryCond() {
		
		Map<String, String> map = new HashMap<String, String>();
		
		String queryReqId = new SimpleDateFormat("yyyyMMddHHmmssSSS").format(new Date());
		map.put("queryReqId", 		queryReqId); //必填，查询请求流水号，只允许使用字母、数字、_、-，每个商户提交的流水号，必须在自身商户号下唯一
		map.put("returnReqId", 		"20211108152033761"); //必填，分账回退请求流水号
		map.put("ext1", ""); //非必填，扩展字段1, 预留
		map.put("ext2", ""); //非必填，扩展字段2, 预留
		
		return map;
	}
	

	/**
	 * 步骤二、构建分账回退查询条件并提交到网关接口
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 * @throws Exception 
	 */
	@PostMapping("/doPaymentDealDivRollbackQuery.do")
	public String doPaymentDealDivRollbackQuery(HttpServletRequest request, HttpServletResponse response, Model model) throws Exception {
		
		// 分账回退结果查询服务的url
		String serverUrl = "https://hfgj.testpnr.com/ledger/rollbackQuery.htm";
		// 分账回退结果查询服务的url(生产环境)
//		String serverUrl = "https://global.chinapnr.com/ledger/rollbackQuery.htm";
		
		// 实例化客户端
		GcbPayClient gcbPayClient = new DefaultGcbPayClient(serverUrl, appConfig.getBase64PrivateKey(), appConfig.getBase64PublicKey());
		
		// 构建请求对象
		GcbPaymentDealDivRollbackQueryRequest queryRequest = new GcbPaymentDealDivRollbackQueryRequest();
		
		//必填，会员账号，由我司提供的商户号+01，共13位数字
		queryRequest.setMerchantAcctId(appConfig.getMerchantAcctId()); 
		//必填，终端号，由我司提供的终端号
		queryRequest.setTerminalId(appConfig.getTerminalId());
		//必填，查询请求流水号，只允许使用字母、数字、_、-，每个商户提交的流水号，必须在自身商户号下唯一
		queryRequest.setQueryReqId(request.getParameter("queryReqId"));
		//必填，分账回退请求流水号
		queryRequest.setReturnReqId(request.getParameter("returnReqId"));
		//非必填，扩展字段1, 预留
		queryRequest.setExt1(request.getParameter("ext1"));
		//非必填，扩展字段2, 预留
		queryRequest.setExt2(request.getParameter("ext2"));
		
		// 发送请求
		GcbPaymentDealDivRollbackQueryResponse queryResponse = gcbPayClient.execute(queryRequest);
		logger.info("GcbPaymentDealDivRollbackQueryResponse => " + new String(queryResponse.getResponseBody()));
		
		// 商户业务逻辑处理
		// ......
		
		model.addAttribute("resultMap", queryResponse.getParamMap());
		return "paymentDealDivRollbackQueryResult";
	}
	
}
