package demo.payment.controller;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;

import com.huifu.jh.core.DefaultGcbPayCallbackMessageParser;
import com.huifu.jh.core.DefaultGcbPayClient;
import com.huifu.jh.core.GcbPayCallbackMessageParser;
import com.huifu.jh.core.GcbPayClient;
import com.huifu.jh.payment.model.GcbPaymentDealRefundRequest;
import com.huifu.jh.payment.model.GcbPaymentDealRefundResponse;
import com.huifu.jh.payment.model.GcbPaymentDealRefundResult;

import demo.payment.config.AppConfig;

/**
 * 
 * @author cheng.luo
 * 
 * 单笔交易退款
 *
 */
@Controller
public class PaymentDealRefundController {
	
	private Logger logger = LoggerFactory.getLogger(getClass());
	
	@Autowired
	private AppConfig appConfig;
	
	/**
	 * 步骤一、构造单笔交易退款信息
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 */
	@GetMapping("/showPaymentDealRefundOrder.do")
	public String showPaymentDealRefundOrder(HttpServletRequest request, HttpServletResponse response, Model model) {
		
		Map<String, String> orderInfoMap = this.buildPaymentDealRefundOrder();
		
		model.addAllAttributes(orderInfoMap);
		
		return "paymentDealRefundOrder";
	}
	
	/**
	 * 构造演示信息
	 * @return
	 */
	private Map<String, String> buildPaymentDealRefundOrder() {
		
		Map<String, String> map = new HashMap<String, String>();
		map.put("bgUrl", 		"http://localhost:8080/refundResultBg.do");  	//必填，服务器接受支付结果的后台地址。bgUrl地址公网必须能访问。
		
		String refundOrderId = new SimpleDateFormat("yyyyMMddHHmmssSSS").format(new Date());
		map.put("refundOrderId", 		refundOrderId); 	//必填，商户退款交易订单号，只允许使用字母、数字、_,并以_，字母或数字开头每商户提交的订单号，必须在自身账户交易中唯一
		map.put("originalOrderId", 		"9920211020182024421"); //必填，原交易订单号，只允许使用字母、数字、_,并以_，字母或数字开头每商户提交的订单号，必须在自身账户交易中唯一
		map.put("refundCurrency", 		"CNY"); //必填，退货币种，字符串,3位币别码，详情请参照币别代码附录进行参考
		map.put("refundAmount", 		"100"); //必填，退货金额，整型数字，单位为分。日韩元支持的最小单位为元。。如100元，则填写10000
		map.put("refundDesc", 		""); //非必填，退货说明,字符串
		map.put("ext1", 		""); //非必填，扩展字段1，英文或中文字符串，交易完成后按照原样返回给商户
		map.put("ext2", 		""); //非必填，扩展字段2，英文或中文字符串，交易完成后按照原样返回给商户
		return map;
	}
	

	/**
	 * 步骤二、构建退款订单信息并提交到服务接口
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 * @throws Exception 
	 */
	@PostMapping("/doPaymentDealRefundApply.do")
	public String doPaymentDealRefundApply(HttpServletRequest request, HttpServletResponse response, Model model) throws Exception {
		
		// 单笔交易退款申请服务的url
		String serverUrl = "https://hfgj.testpnr.com/pay/refundreceive.htm";
		// 单笔交易退款申请服务的url(生产环境)
//		String serverUrl = "https://global.chinapnr.com/pay/refundreceive.htm";
		
		// 实例化客户端
		GcbPayClient gcbPayClient = new DefaultGcbPayClient(serverUrl, appConfig.getBase64PrivateKey(), appConfig.getBase64PublicKey());
		
		// 构建请求对象
		GcbPaymentDealRefundRequest refundRequest = new GcbPaymentDealRefundRequest();
		
		//必填，会员账号，由我司提供的商户号+01，共13位数字
		refundRequest.setMerchantAcctId(appConfig.getMerchantAcctId()); 
		//必填，终端号，由我司提供的终端号
		refundRequest.setTerminalId(appConfig.getTerminalId());
		
		//必填，服务器接受支付结果的后台地址。bgUrl地址公网必须能访问。
		refundRequest.setBgUrl(request.getParameter("bgUrl"));
		//必填，商户退款交易订单号，只允许使用字母、数字、_,并以_，字母或数字开头每商户提交的订单号，必须在自身账户交易中唯一
		refundRequest.setRefundOrderId(request.getParameter("refundOrderId"));
		//必填，原交易订单号，只允许使用字母、数字、_,并以_，字母或数字开头每商户提交的订单号，必须在自身账户交易中唯一
		refundRequest.setOriginalOrderId(request.getParameter("originalOrderId"));
		//必填，退货币种，字符串,3位币别码，详情请参照币别代码附录进行参考
		refundRequest.setRefundCurrency(request.getParameter("refundCurrency"));
		//必填，退货金额，整型数字，单位为分。日韩元支持的最小单位为元。。如100元，则填写10000
		refundRequest.setRefundAmount(request.getParameter("refundAmount"));
		//非必填，退货说明,字符串
		refundRequest.setRefundDesc(request.getParameter("refundDesc"));
		//非必填，扩展字段1，英文或中文字符串，交易完成后按照原样返回给商户
		refundRequest.setExt1(request.getParameter("ext1"));
		//非必填，扩展字段2，英文或中文字符串，交易完成后按照原样返回给商户
		refundRequest.setExt2(request.getParameter("ext2"));
		
		// 发送请求
		GcbPaymentDealRefundResponse refundResponse = gcbPayClient.execute(refundRequest);
		logger.info("GcbPaymentDealRefundResponse => " + new String(refundResponse.getResponseBody()));
		
		// 商户业务逻辑处理
		// ......
		
		model.addAttribute("resultMap", refundResponse.getParamMap());
		return "paymentDealRefundApplyResult";
	}
	
	/**
	 * 步骤四、处理bg支付结果通知
	 * @param request
	 * @param response
	 * @param model
	 * @return
	 * @throws Exception 
	 */
	@RequestMapping("/refundResultBg.do")
	public String processBgNotification(HttpServletRequest request, HttpServletResponse response, Model model) throws Exception {
		
		// 实例化通知解析器
		GcbPayCallbackMessageParser gcbMessageParser = new DefaultGcbPayCallbackMessageParser(appConfig.getBase64PublicKey());
		
		// 解析通知结果
		GcbPaymentDealRefundResult result = gcbMessageParser.parse(request, GcbPaymentDealRefundResult.class);
		logger.info("GcbPaymentDealRefundResult =>" + result.getData());
		
		// 根据通知结果进行业务逻辑处理
		// ......
		
		
		// 向服务端返回: <result>1</result> ,表示结果接收成功，无需再通知。
		response.getWriter().write("<result>1</result>");
		
		return null;
	}
}
